SystemOrganization addCategory: #'Pier-Magma'!

!PRContext methodsFor: '*pier-magma' stamp: 'kph 9/14/2006 03:26'!
maPreserialize

"ensure that properties are always zapped before persisting"
	properties := nil.! !

!PRContext methodsFor: '*pier-magma' stamp: 'kph 9/14/2006 03:25'!
maWantsPreSerialization 
	^ true! !

!PRContext methodsFor: '*pier-magma' stamp: 'kph 9/14/2006 03:27'!
postMagmaCopy

	properties := nil.	 
	self setCommand: self command magmaCopy.! !

PRObject subclass: #PRMagmaRepository
	instanceVariableNames: 'store'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Magma'!

!PRMagmaRepository commentStamp: '<historical>' prior: 0!
I am used as the root object in a magma repository. 
I am subclassed from PRObject so that I may have properties.

----
To Create Repository:

There is a GUI provided in the Magma-seaside package which is available within seaside/config to create a repository.

To manually create a repository directory called 'magma-pier' in the default directory:

MagmaRepositoryController create: 'magma-pier' root: PRMagmaRepository new.

Again there is a GUI available in the 'MagmaControlPanel' for persisting PRKernels.
Manually we tell the default repository to persist an existing pier kernel referenced by its name like so.  

| session |
session := MagmaSession openLocal: 'magma-pier'.
session connectAs: 'local-squeak'.
session commit: [ 
PRMagmaRepository persist: (PRKernel instanceNamed: 'home').
].
session disconnectAndClose.

The existing persistence structure is migrated into the magma repository. The framework for acheiving this is provided in the protocol 'translating'. Although at present this does nothing, it is here in order to allow PRMagmaPersistency to tweak the model that it it persisting. For example we may choose to use indexed MagmaCollections to store the history. The Pier framework need not be aware of any Magma specifics, hence the double dispatch of exportToMagma methods which refer back to Magma specific importFrom methods on PRMagmaPersistency.

 

!

!PRMagmaRepository class methodsFor: 'as yet unclassified' stamp: 'kph 8/24/2006 02:51'!
isAbstract

^false! !

!PRMagmaRepository methodsFor: 'as yet unclassified' stamp: 'kph 8/24/2006 05:24'!
magmaControlPanelOn: magmaSession

^PRMagmaControlPanel on: magmaSession! !

!PRMagmaRepository methodsFor: 'as yet unclassified' stamp: 'kph 9/8/2006 14:36'!
persist: kernel  

	 

	 kernel persistency: ( kernel persistency exportToMagma: PRMagmaPersistency new ).

	store at: kernel name put:  kernel persistency.! !

!PRMagmaRepository methodsFor: 'as yet unclassified' stamp: 'kph 9/8/2006 14:02'!
store

^store ifNil: [ store := Dictionary new ]! !

!PRObject methodsFor: '*pier-magma' stamp: 'kph 9/14/2006 02:47'!
magmaCopy

^ self shallowCopy postMagmaCopy! !

!PRCommand methodsFor: '*pier-magma' stamp: 'kph 9/14/2006 02:28'!
postMagmaCopy

	answer ifNotNil: [ answer := answer magmaCopy ].! !

WAComponent subclass: #PRMagmaControlPanel
	instanceVariableNames: 'magmaSession text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Magma'!

!PRMagmaControlPanel class methodsFor: 'repository init' stamp: 'kph 9/9/2006 00:47'!
copy: kernelName to: repositoryPath

"not tested!!"

"assuming that a kernel is available in memory, or online in another repository.
 deep copy this kernel into the new repository.

e.g.
PRMagmaRepository create: 'backup'.
PRMagmaRepository copy: 'pier' to: 'backup'.
"

|  kernel newKernel session |

kernel := PRKernel instanceNamed: kernelName.

session := self classMagmaSession openLocal:repositoryPath.
session connectAs: 'copy'.
 
kernel critical: [ newKernel := kernel veryDeepCopy ].

session commit: [ session root store at: newKernel name put: newKernel persistency].

session disconnect ;
		closeRepository

"
 
"! !

!PRMagmaControlPanel class methodsFor: 'as yet unclassified' stamp: 'kph 8/24/2006 01:56'!
magmaRootClass

^ PRMagmaRepository ! !

!PRMagmaControlPanel class methodsFor: 'as yet unclassified' stamp: 'kph 9/9/2006 00:48'!
magmaRootDescription

^ 'The Pier Content Management System uses a PRMagmaRepository as a root object. This maintains a Dictionary of PRMagmaPersistency''s, and a Dictionary of properties'! !

!PRMagmaControlPanel class methodsFor: 'as yet unclassified' stamp: 'kph 8/24/2006 02:31'!
on: magmaSession

^self new
	magmaSession: magmaSession;
	yourself.! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/9/2006 14:24'!
deleteKernel: k

Warning signal: 'are you sure you wish to delete kernel: ', k name .

PRKernel instances remove: k ! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/9/2006 14:33'!
loadKernelFromRepository: aKernel

PRKernel instances detect: [ :k | k name = aKernel name ] ifNone: [ ^PRKernel instances add: aKernel ].

self inform: 'Kernel named: ', aKernel name, ' already exists.'.! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 8/24/2006 02:59'!
magmaSession: ms

magmaSession := ms! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 8/24/2006 04:03'!
persist: k

magmaSession root persist: k! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/9/2006 14:49'!
renderContentOn: html


html table: [
	html tableRow: [  
	     html tableData: [
			self renderOn: html table: 'image' with: PRKernel instances using: [:item | self renderKernelInstance: item on: html ].
			
			html textInput size: 10;
				 on: #text of: self. 
			html submitButton callback: [ PRKernel named: self text ] ; text: 'new kernel'.
		].
		html tableData: [
			self renderOn: html table: 'repository' with: self repository store using: [:item | self renderRepository: item on: html ].
		].
	]
]
! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 8/21/2006 04:53'!
rendererClass

^ WARenderCanvas.! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/9/2006 14:23'!
renderKernelInstance: k on: html

html tableData: [ html text: k name ].

html tableData: [ 
	
	html text: '(',k persistency name, ')'.

].
html tableData: [ 
		html small: [ html anchor callback: [ self deleteKernel: k] ; text: '(delete)' ] 
].
html tableData: [ 

(k persistency class ~= PRMagmaPersistency)
 	ifTrue: [
		html submitButton callback: [ self persist: k ] ; text: 'Persist' 
	]

]! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 8/21/2006 04:54'!
renderOn: html table: title with: items using: aBlock

html table: [

	html tableRow: [
		html tableData: [
		html tableHeading: [ html text: title ].
].
	].

	items do: [ :item | 

		html tableRow: [
			 aBlock value: item  
		].

	].

]! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/9/2006 14:36'!
renderRepository: aPersistency on: html

html tableData: [ html text: aPersistency kernel name   ].

html tableData: [ 

	PRKernel instances detect: [ :k | k name = aPersistency kernel name ] 
	ifNone: [  
		html submitButton callback: [self loadKernelFromRepository: aPersistency kernel]; text: 'load'.
	].	
]

! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 8/24/2006 03:00'!
repository

^magmaSession root! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/9/2006 14:20'!
text

^text! !

!PRMagmaControlPanel methodsFor: 'as yet unclassified' stamp: 'kph 9/9/2006 14:21'!
text:   aString

text := aString! !

!WAMagmaSession methodsFor: '*pier-magma' stamp: 'kph 9/8/2006 16:07'!
responseForRequest: aRequest
	
magmaSession ifNotNil: [ magmaSession abort. ].

^super responseForRequest: aRequest! !

PRImagePersistency subclass: #PRMagmaPersistency
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Magma'!

!PRMagmaPersistency commentStamp: 'kph 9/14/2006 03:33' prior: 0!
I am stored in the PRMagmaRepository as the base object for each PRKernel.
I store the current #kernel
I store a historical #snapshot
I store a log of changes, a collection of #contexts, whose commands can be replayed to obtain a complete history.

----
In Pier, PRContext objects may cache #properties in relation to the current seaside display context session and components. Persisting these runtime non-model objects is not a good idea since they may include references to the current magma session which will break the persistency. To avoid this we send #magmaCopy to all instances of PRContext, before saving them into the log of #contexts. This also applies to the 'context's command's answer context'.

If one of these persisted contexts is used and by chance has its properties repopulated, then PRContext-#maPreSerialize zaps #properties as well before the object would be serialised and persisted again. 



!

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 9/11/2006 01:34'!
doing: aCommand critical: aBlock

"this is the essential 'write' interface from pier to magma."

WACurrentSession value commit: [ self kernel critical: aBlock ].! !

!PRMagmaPersistency methodsFor: 'translation' stamp: 'kph 9/8/2006 14:04'!
exportToMagma: m

"already a magma persistence strucure"

^self! !

!PRMagmaPersistency methodsFor: 'translation' stamp: 'kph 8/15/2006 05:20'!
importFromPRPersistency: persistency

"this is our opportunity to munge datastructures into magma-useful things, such as MagmaCollections for historical records etc."

^self! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 9/14/2006 02:11'!
log: aContext

	 contexts add: aContext magmaCopy ! !

!PRMagmaPersistency methodsFor: 'transactions' stamp: 'kph 9/9/2006 00:58'!
resolveKernelFor: client

"I am called with a client session, I check that I am in the repository under my kernel's name
if the persistency found is not me then I update the image to refer to the repository instance in future"
 
| persisted |

persisted := client session root store 
			at: (self kernel name) 
			ifAbsent: [ self inform: ('not found in database: ', self kernel name). self ].

persisted ~= self ifTrue: [  
	PRKernel instances remove: (self kernel).
	PRKernel instances add: persisted kernel.
].

^persisted kernel! !
